
import sys
import time
import random
import math
import copy
import random

import pygame

import screen
import resources
import main_menu
import level_loader
import achivements

FEATHER_CYCLE_SPEED = math.pi #radians/second
FEATHER_CYCLE_SIZE_X = 30 #pixels
FEATHER_CYCLE_SIZE_Y = 10 #pixels
GROUND_Y_POSITION = 0

class Colors:
	BLUE = 1
	PINK = 2
	GREEN = 3

class Feather:
	blue_image = resources.load_image("level","blue_feather.png")
	pink_image = resources.load_image("level","pink_feather.png")
	def __init__(self, color):
		self.color = color
	def draw(self, screen, position_x, position_y):
		if self.color == Colors.BLUE:
			image_to_draw = Feather.blue_image
		elif self.color == Colors.PINK:
			image_to_draw = Feather.pink_image
		else:
			image_to_draw = None
		screen.blit(image_to_draw, (position_x, position_y))

class ScreenFeather:
	def __init__(self, feather, position_x, position_y):
		self.feather = feather
		self.real_position_x = position_x
		self.real_position_y = position_y
		self.start_time = time.time()
		self.calc_position()
	def calc_position(self):
		feather_time = time.time()-self.start_time
		self.position_x = self.real_position_x + math.cos(FEATHER_CYCLE_SPEED * feather_time) * FEATHER_CYCLE_SIZE_X
		self.position_y = self.real_position_y + math.sin(FEATHER_CYCLE_SPEED * feather_time) * FEATHER_CYCLE_SIZE_Y
	def move(self, time_passed):
		self.real_position_y -= 55 * time_passed
		self.calc_position()
	def draw(self, screen, position_view_x, position_view_y):
		self.feather.draw(screen, self.position_x+position_view_x-18, -self.position_y+position_view_y-30)

class FeatherCollection:
	def __init__(self):
		self.feathers = []
	def count(self):
		return len(self.feathers)
	def append(self, feather):
		self.feathers.append(feather)
	def pop(self):
		first = self.feathers[0]
		self.feathers = self.feathers[1:]
		return first
	def draw(self, screen):
		for i in xrange(len(self.feathers)):
			feather_relative_position = 5*i + 10*(i/5)
			self.feathers[i].draw(screen,700-feather_relative_position, 20)
	def get_blue_feathers(self):
		return sum(feather.color == Colors.BLUE for feather in self.feathers)
	def get_pink_feathers(self):
		return sum(feather.color == Colors.PINK for feather in self.feathers)

class Bird:
	blue_images = [resources.load_image("level","birds","bird_blue_1.png"), resources.load_image("level","birds","bird_blue_2.png")]
	pink_images = [resources.load_image("level","birds","bird_pink_1.png"), resources.load_image("level","birds","bird_pink_2.png")]
	naked_images = [resources.load_image("level","birds","bird_naked_1.png"), resources.load_image("level","birds","bird_naked_2.png")]
	def __init__(self, position_x, position_y, color, speed):
		self.position_x = position_x
		self.position_y = position_y
		self.color = color
		self.speed = speed
		
		self.is_hit = False
		self.last_time_the_bird_hit_icarus = time.time()
	def move(self, time_passed):
		self.position_x -= self.speed * time_passed
		if self.is_hit:
			self.position_y -= 30 * time_passed
	def draw(self, screen, position_view_x, position_view_y):
		index = int(time.time()*5) %2
		if self.is_hit:
			image_to_draw = Bird.naked_images[index]
		else:
			if self.color == Colors.BLUE:
				image_to_draw = Bird.blue_images[index]
			elif self.color == Colors.PINK:
				image_to_draw = Bird.pink_images[index]
			else:
				image_to_draw = None
		screen.blit(image_to_draw,
					[self.position_x+position_view_x-49,
					 -self.position_y+position_view_y-48])

class Platform:
	image = resources.load_image("level","island.png")
	end_flag_image = resources.load_image("level","end.png")
	def __init__(self, position_x, has_flag):
		self.position_x = position_x
		self.has_flag = has_flag
	def draw(self, screen, position_view_x, position_view_y):
		screen.blit(Platform.image,[self.position_x+position_view_x,40+ position_view_y])
		if self.has_flag:
			screen.blit(Platform.end_flag_image,[self.position_x+position_view_x+130,-80+ position_view_y])
	def is_icarus_on(self, icarus_position_x):
		return icarus_position_x >= self.position_x+25 and icarus_position_x <= 340 +self.position_x


icarus_images = [resources.load_image("level","icarus","icarus%d.png" % i) for i in range(1,4)]
icarus_hand_up_images = [resources.load_image("level","icarus","icarus_hand_up.png")]
icarus_hit_images = [resources.load_image("level","icarus","icarus_hit.png")]
icarus_scared_images = [resources.load_image("level","icarus","icarus_scared%d.png" % i) for i in range(1,3)]
wing_images = [resources.load_image("level","icarus","wing%d.png" % i) for i in range(1,6)]
wing_images_flipped = [pygame.transform.flip(img, True, False) for img in wing_images]

wing_flap_sound = resources.load_sound("flap.ogg")
win_sound = resources.load_sound("win.ogg")
crow_sound = resources.load_sound("crow.ogg")
ahh_sound = resources.load_sound("aahhzz.ogg")
splash_sound = resources.load_sound("splash.ogg")

sea_image = resources.load_image("level","ocean.png")
sky = resources.load_image("level","sun.png")
game_over_image = resources.load_image("level","game_over.png")
you_win_image = resources.load_image("level","you_win.png")
thermometer_image = resources.load_image("level","thermometer.png")

class Parameters:
	LevelName = "Generic"
	
	GOLDEN_TROPHY_TIME = 25
	SILVER_TROPHY_TIME = 30
	GOLDEN_TROPHY_FEATHERS = 50
	SILVER_TROPHY_FEATHERS = 20
	GOLDEN_TROPHY_HEIGHT = 3000
	SILVER_TROPHY_HEIGHT = 2000
	
	BIRDS_SPEED = 35.0 #pixels/second
	TIME_BETWEEN_BIRDS = 5.0 #seconds
	MAX_LANDING_SPEED = 150 #pixels/second
	ICARUS_GROUND_SPEED = 160 #pixels/second
	ICARUS_HIT_IMAGE_DURATION = 0.8 #seconds
	TIME_BETWEEN_BIRD_HITS = 3.2 #seconds
	GRAVITY = -50.0 #pixels/second^2
	DEAD_GRAVITY = -260
	WING_FLAP_SPEED = 10 #pixels/second/feather
	WORLD_WIDTH = 6*800
	SUN_POSITION = 1300
	MAX_TEMPERATURE_HEIGHT = 1300
	MIN_TEMPERATURE_HEIGHT = 30
	
	WAX_SOFTNESS = 1.0 # feathers / second at full heat
	
	@staticmethod
	def add_init_feathers(feather_collection):
		for i in xrange(9):
			feather_collection.append(Feather(Colors.BLUE))
			feather_collection.append(Feather(Colors.PINK))

	@staticmethod
	def add_init_platforms(platforms):
		platforms.append(Platform(-160,False))
		platforms.append(Platform((5.7)*800,True))
	
	@staticmethod
	def add_init_birds(birds,speed):
		for i in range(15):
			birds.append(Bird(random.randint(1000,4000),random.randint(100,500), random.choice([Colors.BLUE,Colors.PINK]),speed))
	
	@staticmethod
	def add_bird(birds, icarus_position_x, icarus_position_y,speed):
		birds.append(Bird(icarus_position_x+random.randint(900,4000),random.randint(100,400), random.choice([Colors.BLUE,Colors.PINK]),speed))
	
class Level(screen.Screen):
	def __init__(self, parameters):
		screen.Screen.__init__(self)
		self.parameters = parameters
		self.start_time = time.time()
		
		self.is_pressed_right = False
		self.is_pressed_left = False
		self.is_pressed_up = False
		self.is_pressed_down = False
		
		self.screen_feathers = []
		self.feather_collection = FeatherCollection()
		self.birds = []
		self.platforms = []
		self.last_time_icarus_was_hit = 0
		self.last_wing_flap_time = 0
		self.speed_x = 0
		self.speed_y = 0

		self.parameters.add_init_platforms(self.platforms)		
		self.parameters.add_init_feathers(self.feather_collection)
		self.parameters.add_init_birds(self.birds, self.parameters.BIRDS_SPEED)
		
		self.icarus_position_x = 45 # position of center of body
		self.icarus_position_y = 0
		self.max_height = 0
		
		self.time_to_next_bird = time.time() + self.parameters.TIME_BETWEEN_BIRDS
		self.game_over = False
		self.has_won = False
		
		self.wing_strength = 1.0
	
	def start(self):
		pygame.display.set_caption("Icarus - %s" % self.parameters.LevelName)

	def is_on_platform(self):
		if self.icarus_position_y > GROUND_Y_POSITION:
			return False
		for platform in self.platforms:
			if platform.is_icarus_on(self.icarus_position_x):
				return True
		return False
	
	def is_on_flag_platform(self):
		if self.icarus_position_y > GROUND_Y_POSITION:
			return False
		for platform in self.platforms:
			if not platform.has_flag:
				continue
			if platform.is_icarus_on(self.icarus_position_x):
				return True
		return False

	def get_flap_duration(self):
		frames_per_5_seconds = self.feather_collection.get_pink_feathers() + 1
		return 5.0 / frames_per_5_seconds

	def flap_wings(self):
		if self.game_over:
			return
		if time.time() < self.get_flap_duration() + self.last_wing_flap_time:
			return
		if self.is_on_platform() and not self.is_pressed_up:
			return
		if self.is_pressed_up or self.is_pressed_down or self.is_pressed_left or self.is_pressed_right:
			#do the flap:
			wing_flap_sound.play()
			actual_flap_speed = self.feather_collection.get_blue_feathers()*self.parameters.WING_FLAP_SPEED
			self.last_wing_flap_time = time.time()
			
			if self.is_pressed_up:
				self.speed_y += actual_flap_speed
			if self.is_pressed_down:
				self.speed_y -= actual_flap_speed
			if self.is_pressed_right :
				self.speed_x += actual_flap_speed
			if self.is_pressed_left :
				self.speed_x -= actual_flap_speed
	
	def handle_event(self, event):
		screen.Screen.handle_event(self,event)
		
		if not self.game_over:
			if event.type == pygame.KEYDOWN:
				is_pressed = True
			elif event.type == pygame.KEYUP:
				is_pressed = False
			else:
				return
			if event.key == pygame.K_RIGHT:
				self.is_pressed_right = is_pressed
			if event.key == pygame.K_LEFT:
				self.is_pressed_left = is_pressed
			if event.key == pygame.K_UP:
				self.is_pressed_up = is_pressed
			if event.key == pygame.K_DOWN:
				self.is_pressed_down = is_pressed
			
			#do this in addition to do_frame, in order to flap even if there was no frame in between the DOWN and UP events
			self.flap_wings()
		
		if event.type == pygame.KEYDOWN:
			if event.key == pygame.K_RETURN:
				if self.game_over:
					self.end_level = True
					self.next_screen = Level(self.parameters)
				if self.has_won:
					self.end_level = True
					this_index = level_loader.levels.index(self.parameters)
					if this_index+1 == len(level_loader.levels):
						self.next_screen = main_menu.MainMenu()
					else:
						self.next_screen = level_loader.load_level(this_index+1)
	
	def kill_icarus(self):
		self.game_over = True
		self.speed_y = 150
		self.speed_x = -80
	
	def drop_feather(self):
		if self.feather_collection.count() > 0:
			#lose the feather from the bottom, so it will be hard to reach
			self.screen_feathers.append(ScreenFeather(self.feather_collection.pop(),self.icarus_position_x, self.icarus_position_y-120))

	def do_frame(self, time_passed):
		#flap:
		self.flap_wings()
		
		#move:
		if self.game_over:
			self.icarus_position_x += self.speed_x * time_passed
		elif self.has_won:
			pass
		elif self.is_on_platform():
			self.speed_x = 0
			if self.is_pressed_right:
				self.icarus_position_x += self.parameters.ICARUS_GROUND_SPEED * time_passed
			if self.is_pressed_left:
				self.icarus_position_x -= self.parameters.ICARUS_GROUND_SPEED * time_passed
		else:
			self.speed_x *= math.exp(-1*time_passed)
			self.icarus_position_x += self.speed_x * time_passed
		
		#fall:
		if self.game_over:
			if self.icarus_position_y < -500:
				self.speed_x = 0
			else:
				self.speed_y += self.parameters.DEAD_GRAVITY * time_passed
				self.icarus_position_y += self.speed_y * time_passed
		
		elif self.is_on_platform() and self.speed_y<=0:
			if self.speed_y < -self.parameters.MAX_LANDING_SPEED:
				self.kill_icarus()
			else:
				# standing on the flatform
				self.icarus_position_y = GROUND_Y_POSITION
				self.speed_y = 0
		else:
			#free falling
			self.speed_y += self.parameters.GRAVITY * time_passed
			self.icarus_position_y += self.speed_y * time_passed
			if self.icarus_position_y > self.max_height:
				self.max_height = self.icarus_position_y

		#falling to the water:
		if (not self.game_over) and (not self.has_won):
			if self.icarus_position_y < -50:
				splash_sound.play()
				self.kill_icarus()
		
		#create birds
		if time.time() > self.time_to_next_bird :
			self.parameters.add_bird(self.birds, self.icarus_position_x, self.icarus_position_y, self.parameters.BIRDS_SPEED)
			self.time_to_next_bird += self.parameters.TIME_BETWEEN_BIRDS
		
		#checking hitting a bird
		if not self.has_won:
			for bird in self.birds:
				#relative position from icarus:
				rel_x = self.icarus_position_x - bird.position_x
				rel_y = self.icarus_position_y - bird.position_y
				#from above:
				#50=size of rect around the centers of icarus and the bird
				#-90=addition of length from center to feet of icarus
				if abs(rel_x) < 50:
					if abs(rel_y-90) < 50:
						if not bird.is_hit: #makes sure only one feather is created
							bird.is_hit = True
							crow_sound.play()
							self.screen_feathers.append(ScreenFeather(Feather(bird.color),bird.position_x, bird.position_y))
				#from below:
				#this time we take +20 as length from center to head
				if abs(rel_x) < 50:
					if abs(rel_y+20) < 50:
						if time.time() - bird.last_time_the_bird_hit_icarus > self.parameters.TIME_BETWEEN_BIRD_HITS:
							bird.last_time_the_bird_hit_icarus = time.time()
							self.last_time_icarus_was_hit = time.time()
							self.drop_feather()
							ahh_sound.play()
		
		#checking if catching a feather
		if not self.has_won:
			for feather in copy.copy(self.screen_feathers):
				#relative position from icarus:
				rel_x = self.icarus_position_x - feather.position_x
				rel_y = self.icarus_position_y - feather.position_y
				if abs(rel_x) < 50:
					if abs(rel_y+20) < 50:
						self.screen_feathers.remove(feather)
						self.feather_collection.append(feather.feather)
		
		for feather in copy.copy(self.screen_feathers):
			if feather.position_y < -100:
				self.screen_feathers.remove(feather)
		for bird in copy.copy(self.birds):
			if bird.position_x < -100:
				self.birds.remove(bird)
		
		for feather in self.screen_feathers:
			feather.move(time_passed)
		
		for bird in self.birds:
			bird.move(time_passed)
		
		#calculate temperature
		self.temperature = (self.icarus_position_y-self.parameters.MIN_TEMPERATURE_HEIGHT)/float(self.parameters.MAX_TEMPERATURE_HEIGHT-self.parameters.MIN_TEMPERATURE_HEIGHT)
		if self.temperature < 0:
			self.temperature = 0
		if self.temperature > 1:
			self.temperature = 1
		
		#heat - feathers fall
		if not self.has_won:
			self.wing_strength -= self.temperature * time_passed * self.parameters.WAX_SOFTNESS
			if self.wing_strength <= 0:
				self.wing_strength += 1
				self.drop_feather()
			if self.feather_collection.count() == 0:
				self.wing_strength = 1
		
		if (not self.has_won) and self.is_on_flag_platform():
			this_index = level_loader.levels.index(self.parameters)
			achivements.won_level(
				this_index,
				self.parameters,
				time.time() - self.start_time,
				len(self.feather_collection.feathers),
				self.max_height)
			self.has_won = True
			win_sound.play()
	
	def draw_icarus(self, screen, position_view_x, position_view_y):
		#wing flapping:
		flap_time = (time.time() - self.last_wing_flap_time)/self.get_flap_duration()
		if flap_time >= 1:
			index = 0
		else:
			image_set_count = len(wing_images) * 2 - 1
			index = int(flap_time * image_set_count)
			if index >= len(wing_images):
				index = 2*len(wing_images) - index - 1
		screen.blit(wing_images[index],
			[int(self.icarus_position_x+position_view_x-87),int(-self.icarus_position_y+position_view_y-52)])
		screen.blit(wing_images_flipped[index],
			[int(self.icarus_position_x+position_view_x-9),int(-self.icarus_position_y+position_view_y-52)])
		
		#icarus himself:
		if time.time()-self.last_time_icarus_was_hit < self.parameters.ICARUS_HIT_IMAGE_DURATION:
			chosen_icarus_image_set = icarus_hit_images
		elif self.game_over:
			chosen_icarus_image_set = icarus_hit_images
		elif self.has_won:
			chosen_icarus_image_set = icarus_hand_up_images
		elif self.feather_collection.get_blue_feathers() < 8 or self.feather_collection.get_pink_feathers() < 8:
			chosen_icarus_image_set = icarus_scared_images
		else:
			chosen_icarus_image_set = icarus_images
		
		chosen_icarus_image = chosen_icarus_image_set[int(time.time()) % len(chosen_icarus_image_set)]
		screen.blit(chosen_icarus_image,[int(self.icarus_position_x+position_view_x-87),int(-self.icarus_position_y+position_view_y-71)])
	
	def draw_image(self, screen, image):
		screen.blit(image, [
			(800-image.get_width())/2,
			(600-image.get_height())/2])
	
	def draw(self, screen):
		# ################## #
		#      DRAWING:      #
		# ################## #
		
		#how much to move everything in the world:
		position_view_x = self.screen_width/2 - self.icarus_position_x
		position_view_y = self.screen_height/2 + self.icarus_position_y
		
		if position_view_y <0 and False:
			position_view_y = 0
		
		position_view_x = min(position_view_x, 0)
		position_view_x = max(position_view_x, self.screen_width-self.parameters.WORLD_WIDTH)
		position_view_y = max(position_view_y, self.screen_height/2)
		
		screen.fill ((0,255,255))
		
		#sea
		sea_width = sea_image.get_width()
		sea_height = sea_image.get_height()
		screen.blit(sea_image,[position_view_x%sea_width-sea_width,position_view_y+(300-sea_height)])
		screen.blit(sea_image,[position_view_x%sea_width          ,position_view_y+(300-sea_height)])
		
		#sun, up at the sky
		screen.blit(sky,[60,position_view_y-self.parameters.SUN_POSITION])
		
		#islands
		for platform in self.platforms:
			platform.draw(screen, position_view_x, position_view_y)	
		
		#draw birds:
		for bird in self.birds:
			bird.draw(screen, position_view_x, position_view_y)
		
		#feathers display:
		self.feather_collection.draw(screen)
		
		#icarus:
		self.draw_icarus(screen, position_view_x, position_view_y)

		if self.game_over:
			self.draw_image(screen, game_over_image)
		if self.has_won:
			self.draw_image(screen, you_win_image)
		
		#screen feathers:
		for feather in self.screen_feathers:
			feather.draw(screen, position_view_x, position_view_y)
		
		#thermometer
		temperature_height = 1-self.temperature
		pygame.draw.rect(screen,(255,0,0),(753,100+temperature_height*140,40,180-temperature_height*140))
		screen.blit(thermometer_image, [750,100])
		