import sys

import pygame

import screen
import resources
import level_loader
import achivements

background = resources.load_image("menu","background.png")
select_image = resources.load_image("menu","select.png")
next_level = resources.load_image("menu","right_wing.png")
prev_level = pygame.transform.flip(next_level, True, False)

trophy_images = [resources.load_image("trophys","trophy_%s.png"%name) for name in ["black", "bronze", "silver", "gold"]]

wings_distance_from_edge = 50
wings_y_position = 440

class LevelSelection(screen.Screen):
	
	def __init__(self):
		screen.Screen.__init__(self)
		self.selected_item = 0
	
	def start(self):
		pygame.display.set_caption("Icarus - Level selection")
		self.font = pygame.font.Font(None, 80)
		self.smaller_font = pygame.font.Font(None, 38)
		self.even_smaller_font = pygame.font.Font(None, 30)
	
	def load_level(self):
		self.end_level = True
		self.next_screen = level_loader.load_level(self.selected_item)
	def next_level(self):
		self.selected_item += 1
		if self.selected_item >= len(level_loader.levels):
			self.selected_item = len(level_loader.levels) - 1
	def prev_level(self):
		self.selected_item -= 1
		if self.selected_item < 0:
			self.selected_item = 0
	
	def handle_event(self, event):
		screen.Screen.handle_event(self,event)
		
		if event.type == pygame.KEYDOWN and event.key == pygame.K_RETURN:
			self.load_level()
		if event.type == pygame.KEYDOWN and event.key == pygame.K_RIGHT:
			self.next_level()
		if event.type == pygame.KEYDOWN and event.key == pygame.K_LEFT:
			self.prev_level()
		
		if event.type == pygame.MOUSEBUTTONDOWN:
			if event.pos[1] >= wings_y_position and event.pos[1] <= wings_y_position+next_level.get_height():
				if event.pos[0] <= wings_distance_from_edge + next_level.get_width():
					self.prev_level()
				elif event.pos[0] >= 800 - ( wings_distance_from_edge + next_level.get_width() ) :
					self.next_level()
				else:
					self.load_level()
	
	def do_frame(self, time_passed):
		pass
	
	def draw(self, screen):
		screen.blit(background,[0,0])
		
		#level name:
		text = self.font.render(level_loader.levels[self.selected_item].LevelName,True, (0,0,0))
		screen.blit(text, ((800-text.get_width())/2,30))
		
		trophys = achivements.get_trophys(self.selected_item)
		for i in range(len(trophys)):
			height = 100 + i*110
			screen.blit(trophy_images[trophys[i][0]], (60, height))
			text = self.smaller_font.render(trophys[i][1], True, (0,0,0))
			screen.blit(text, (200,height+30))
			text = self.even_smaller_font.render(trophys[i][2], True, (0,0,0))
			screen.blit(text, (200,height+60))
		
		#buttons:
		screen.blit(next_level,[800-(wings_distance_from_edge+next_level.get_width()),wings_y_position])
		screen.blit(prev_level,[wings_distance_from_edge,wings_y_position])
		screen.blit(select_image,[(800-select_image.get_width())/2,wings_y_position])
		
		
		