import sys

import pygame

import screen
import resources
import level_loader
import level_selection

background = resources.load_image("menu","background.png")
icarus = resources.load_image("menu","icarus.png")

start_images = [resources.load_image("menu","start_game.png"),resources.load_image("menu","start_game_big.png")]
select_level_images = [resources.load_image("menu","select_level.png"),resources.load_image("menu","select_level_big.png")]
quit_images = [resources.load_image("menu","quit.png"),resources.load_image("menu","quit_big.png")]

buttons_images = [start_images, select_level_images, quit_images]
buttons_positions = [260,360,460]

class MainMenu(screen.Screen):	
	def __init__(self):
		screen.Screen.__init__(self)
		self.selected_item = None
	
	def start(self):
		pygame.display.set_caption("Icarus")
	
	def do_action(self, i):
		if i == 0:
			self.end_level = True
			self.next_screen = level_loader.load_level(0)
		if i == 1:
			self.end_level = True
			self.next_screen = level_selection.LevelSelection()
		if i == 2:
			sys.exit()
	
	def get_item(self, position):
		for i in range(len(buttons_images)):
			button_position = buttons_positions[i]
			button_image = buttons_images[i][0]
			placement = (800-button_image.get_width()) / 2
			if position[0] >= placement and position[0] <= 800-placement:
				if position[1] >= button_position and position[1] <= button_position + button_image.get_height():
					return i
		return None
	
	def handle_event(self, event):
		screen.Screen.handle_event(self,event)
		
		if event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
			sys.exit()
		
		if event.type == pygame.MOUSEMOTION:
			self.selected_item = self.get_item(event.pos)
		if event.type == pygame.MOUSEBUTTONDOWN:
			self.selected_item = self.get_item(event.pos)
			self.do_action(self.selected_item)
		
		if event.type == pygame.KEYDOWN and event.key == pygame.K_RETURN:
			self.do_action(self.selected_item)
		if event.type == pygame.KEYDOWN and event.key == pygame.K_DOWN:
			if self.selected_item == None:
				self.selected_item = 0
			else:
				self.selected_item = (self.selected_item+1) % len(buttons_images)
		if event.type == pygame.KEYDOWN and event.key == pygame.K_UP:
			if self.selected_item == None:
				self.selected_item = len(buttons_images)-1
			else:
				self.selected_item = (self.selected_item+len(buttons_images)-1) % len(buttons_images)
	
	def do_frame(self, time_passed):
		pass
	
	def draw(self, screen):
		screen.blit(background,[0,0])
		
		screen.blit(icarus, [(800-icarus.get_width())/2,10])
		
		for i in range(len(buttons_images)):
			if i == self.selected_item:
				button_image = buttons_images[i][1]
				placement = (800-button_image.get_width()) / 2
				placement_y_diff = buttons_images[i][1].get_height() - buttons_images[i][0].get_height()
				screen.blit(button_image, (placement, buttons_positions[i]-placement_y_diff/2))
			else:
				button_image = buttons_images[i][0]
				placement = (800-button_image.get_width()) / 2
				screen.blit(button_image, (placement, buttons_positions[i]))
